@@ -74,10 +74,13 @@ static void init_rsqrt(void)
     int i;
     for (i = 0; i < DUMTAB1SIZE; i++)
     {
-        float f;
+        union {
+          float f;
+          long l;
+        } u;
         int32_t l = (i ? (i == DUMTAB1SIZE-1 ? DUMTAB1SIZE-2 : i) : 1)<< 23;
-        *(int32_t *)(&f) = l;
-        rsqrt_exptab[i] = 1./sqrt(f);   
+        u.l = l;
+        rsqrt_exptab[i] = 1./sqrt(u.f);   
     }
     for (i = 0; i < DUMTAB2SIZE; i++)
     {
@@ -90,30 +93,30 @@ static void init_rsqrt(void)
 
 t_float q8_rsqrt(t_float f0)
 {
-    float f = (float)f0;
-    long l = *(long *)(&f);
-    if (f < 0) return (0);
-    else return (rsqrt_exptab[(l >> 23) & 0xff] *
-            rsqrt_mantissatab[(l >> 13) & 0x3ff]);
+    union {
+      float f;
+      long l;
+    } u;
+    u.f=f0;
+    if (u.f < 0) return (0);
+    else return (rsqrt_exptab[(u.l >> 23) & 0xff] *
+            rsqrt_mantissatab[(u.l >> 13) & 0x3ff]);
 }
 
 t_float q8_sqrt(t_float f0)
 {
-    float f = (float)f0;
-    long l = *(long *)(&f);
-    if (f < 0) return (0);
-    else return (f * rsqrt_exptab[(l >> 23) & 0xff] *
-            rsqrt_mantissatab[(l >> 13) & 0x3ff]);
+    union {
+      float f;
+      long l;
+    } u;
+    u.f=f0;
+    if (u.f < 0) return (0);
+    else return (u.f * rsqrt_exptab[(u.l >> 23) & 0xff] *
+            rsqrt_mantissatab[(u.l >> 13) & 0x3ff]);
 }
 
-    /* the old names are OK unless we're in IRIX N32 */
-
-#ifndef N32
 t_float qsqrt(t_float f) {return (q8_sqrt(f)); }
 t_float qrsqrt(t_float f) {return (q8_rsqrt(f)); }
-#endif
-
-
 
 typedef struct sigrsqrt
 {
