@@ -42,7 +42,6 @@ desktops because the borders have both window title area and menus. */
 extern t_pd *newest;
 t_class *canvas_class;
 int canvas_dspstate;                /* whether DSP is on or off */  
-t_canvas *canvas_editing;           /* last canvas to start text edting */ 
 t_canvas *canvas_whichfind;         /* last canvas we did a find in */ 
 t_canvas *canvas_list;              /* list of all root canvases */
 
@@ -55,6 +54,8 @@ void canvas_reflecttitle(t_canvas *x);
 static void canvas_addtolist(t_canvas *x);
 static void canvas_takeofflist(t_canvas *x);
 static void canvas_pop(t_canvas *x, t_floatarg fvis);
+static void canvas_bind(t_canvas *x);
+static void canvas_unbind(t_canvas *x);
 
 /* --------- functions to handle the canvas environment ----------- */
 
@@ -205,11 +206,9 @@ void canvas_makefilename(t_canvas *x, char *file, char *result, int resultsize)
 
 void canvas_rename(t_canvas *x, t_symbol *s, t_symbol *dir)
 {
-    if (strcmp(x->gl_name->s_name, "Pd"))
-        pd_unbind(&x->gl_pd, canvas_makebindsym(x->gl_name));
+    canvas_unbind(x);
     x->gl_name = s;
-    if (strcmp(x->gl_name->s_name, "Pd"))
-        pd_bind(&x->gl_pd, canvas_makebindsym(x->gl_name));
+    canvas_bind(x);
     if (x->gl_havewindow)
         canvas_reflecttitle(x);
     if (dir && dir != &s_)
@@ -379,8 +378,7 @@ t_canvas *canvas_new(void *dummy, t_symbol *sel, int argc, t_atom *argv)
     x->gl_owner = owner;
     x->gl_name = (*s->s_name ? s : 
         (canvas_newfilename ? canvas_newfilename : gensym("Pd")));
-    if (strcmp(x->gl_name->s_name, "Pd"))
-        pd_bind(&x->gl_pd, canvas_makebindsym(x->gl_name));
+    canvas_bind(x);
     x->gl_loading = 1;
     x->gl_goprect = 0;      /* no GOP rectangle unless it's turned on later */
         /* cancel "vis" flag if we're a subpatch of an
@@ -479,9 +477,8 @@ t_glist *glist_addglist(t_glist *g, t_symbol *sym,
     x->gl_screeny1 = GLIST_DEFCANVASYLOC;
     x->gl_screenx2 = 450;
     x->gl_screeny2 = 300;
-    if (strcmp(x->gl_name->s_name, "Pd"))
-        pd_bind(&x->gl_pd, canvas_makebindsym(x->gl_name));
     x->gl_owner = g;
+    canvas_bind(x);
     x->gl_isgraph = 1;
     x->gl_goprect = 0;
     x->gl_obj.te_binbuf = binbuf_new();
@@ -561,6 +558,21 @@ t_symbol *canvas_makebindsym(t_symbol *s)
     return (gensym(buf));
 }
 
+    /* functions to bind and unbind canvases to symbol "pd-blah".  As
+    discussed on Pd dev list there should be a way to defeat this for
+    abstractions.  (Claude Heiland et al. Aug 9 2013) */
+static void canvas_bind(t_canvas *x)
+{
+    if (strcmp(x->gl_name->s_name, "Pd"))
+        pd_bind(&x->gl_pd, canvas_makebindsym(x->gl_name));
+}
+
+static void canvas_unbind(t_canvas *x)
+{
+    if (strcmp(x->gl_name->s_name, "Pd"))
+        pd_unbind(&x->gl_pd, canvas_makebindsym(x->gl_name));
+}
+
 void canvas_reflecttitle(t_canvas *x)
 {
     char namebuf[MAXPDSTRING];
@@ -712,8 +724,6 @@ void canvas_free(t_canvas *x)
     t_gobj *y;
     int dspstate = canvas_suspend_dsp();
     canvas_noundo(x);
-    if (canvas_editing == x)
-        canvas_editing = 0;
     if (canvas_whichfind == x)
         canvas_whichfind = 0;
     glist_noselect(x);
@@ -723,8 +733,7 @@ void canvas_free(t_canvas *x)
         canvas_vis(x, 0);
     if (x->gl_editor)
         canvas_destroy_editor(x);   /* bug workaround; should already be gone*/
-    if (strcmp(x->gl_name->s_name, "Pd"))
-        pd_unbind(&x->gl_pd, canvas_makebindsym(x->gl_name));
+    canvas_unbind(x);
 
     if (x->gl_env)
     {
@@ -977,62 +986,14 @@ static void canvas_rename_method(t_canvas *x, t_symbol *s, int ac, t_atom *av)
     else canvas_rename(x, gensym("Pd"), 0);
 }
 
-/* ------------------ table ---------------------------*/
-
-static int tabcount = 0;
-
-static void *table_new(t_symbol *s, t_floatarg f)
-{
-    t_atom a[9];
-    t_glist *gl;
-    t_canvas *x, *z = canvas_getcurrent();
-    if (s == &s_)
-    {
-         char  tabname[255];
-         t_symbol *t = gensym("table"); 
-         sprintf(tabname, "%s%d", t->s_name, tabcount++);
-         s = gensym(tabname); 
-    }
-    if (f <= 1)
-        f = 100;
-    SETFLOAT(a, 0);
-    SETFLOAT(a+1, GLIST_DEFCANVASYLOC);
-    SETFLOAT(a+2, 600);
-    SETFLOAT(a+3, 400);
-    SETSYMBOL(a+4, s);
-    SETFLOAT(a+5, 0);
-    x = canvas_new(0, 0, 6, a);
-
-    x->gl_owner = z;
-
-        /* create a graph for the table */
-    gl = glist_addglist((t_glist*)x, &s_, 0, -1, (f > 1 ? f-1 : 1), 1,
-        50, 350, 550, 50);
-
-    graph_array(gl, s, &s_float, f, 0);
-
-    canvas_pop(x, 0); 
-
-    return (x);
-}
 
     /* return true if the "canvas" object is an abstraction (so we don't
-    save its contents, fogr example.)  */
+    save its contents, for example.)  */
 int canvas_isabstraction(t_canvas *x)
 {
     return (x->gl_env != 0);
 }
 
-    /* return true if the "canvas" object is a "table". */
-int canvas_istable(t_canvas *x)
-{
-    t_atom *argv = (x->gl_obj.te_binbuf? binbuf_getvec(x->gl_obj.te_binbuf):0);
-    int argc = (x->gl_obj.te_binbuf? binbuf_getnatom(x->gl_obj.te_binbuf) : 0);
-    int istable = (argc && argv[0].a_type == A_SYMBOL &&
-        argv[0].a_w.w_symbol == gensym("table"));
-    return (istable);
-}
-
     /* return true if the "canvas" object should be treated as a text
     object.  This is true for abstractions but also for "table"s... */
 /* JMZ: add a flag to gop-abstractions to hide the title */
@@ -1168,8 +1129,6 @@ and can perhaps regard this is a design flaw in jack that we're working around
 here.  The function audio_shouldkeepopen() is provided by s_audio.c to tell
 us that we should elide the step of closing audio when DSP is turned off.*/
 
-int audio_shouldkeepopen( void);
-
 void glob_dsp(void *dummy, t_symbol *s, int argc, t_atom *argv)
 {
     int newstate;
@@ -1441,11 +1400,37 @@ int canvas_open(t_canvas *x, const char *name, const char *ext,
 static void canvas_f(t_canvas *x, t_symbol *s, int argc, t_atom *argv)
 {
     static int warned;
-    if (!warned)
+    t_gobj *g, *g2;
+    t_object *ob;
+    if (argc > 1 && !warned)
     {
         post("** ignoring width or font settings from future Pd version **");
         warned = 1;
     }
+    if (!x->gl_list)
+        return;
+    for (g = x->gl_list; g2 = g->g_next; g = g2)
+        ;
+    if (ob = pd_checkobject(&g->g_pd))
+    {
+        ob->te_width = atom_getfloatarg(0, argc, argv);
+        if (glist_isvisible(x))
+        {
+            gobj_vis(g, x, 0);
+            gobj_vis(g, x, 1);
+        }
+    }
+}
+
+extern t_class *array_define_class;     /* LATER datum class too */
+
+    /* check if a pd can be treated as a glist - true if we're of any of
+    the glist classes, which all have 'glist' as the first item in struct */
+t_glist *pd_checkglist(t_pd *x)
+{
+    if (*x == canvas_class || *x == array_define_class)
+        return ((t_canvas *)x);
+    else return (0);
 }
 
 /* ------------------------------- setup routine ------------------------ */
@@ -1566,11 +1551,6 @@ void g_canvas_setup(void)
     class_addmethod(canvas_class, (t_method)canvas_rename_method,
         gensym("rename"), A_GIMME, 0);
 
-/*---------------------------- tables -- GG ------------------- */
-
-    class_addcreator((t_newmethod)table_new, gensym("table"),
-        A_DEFSYM, A_DEFFLOAT, 0);
-
 /*---------------------------- declare ------------------- */
     declare_class = class_new(gensym("declare"), (t_newmethod)declare_new,
         (t_method)declare_free, sizeof(t_declare), CLASS_NOINLET, A_GIMME, 0);
@@ -1585,3 +1565,25 @@ void g_canvas_setup(void)
     g_editor_setup();
     g_readwrite_setup();
 }
+
+    /* functions to add basic gui (e.g., clicking but not editing) to things
+    based on canvases that aren't editable, like "array define" object */
+void canvas_editor_for_class(t_class *c);
+void g_graph_setup_class(t_class *c);
+void canvas_readwrite_for_class(t_class *c);
+
+void canvas_add_for_class(t_class *c)
+{
+    class_addmethod(c, (t_method)canvas_restore,
+        gensym("restore"), A_GIMME, 0);
+    class_addmethod(c, (t_method)canvas_click,
+        gensym("click"), A_FLOAT, A_FLOAT, A_FLOAT, A_FLOAT, A_FLOAT, 0);
+    class_addmethod(c, (t_method)canvas_dsp, gensym("dsp"), 0);
+    class_addmethod(c, (t_method)canvas_map,
+        gensym("map"), A_FLOAT, A_NULL);
+    class_addmethod(c, (t_method)canvas_setbounds,
+        gensym("setbounds"), A_FLOAT, A_FLOAT, A_FLOAT, A_FLOAT, A_NULL);
+    canvas_editor_for_class(c);
+    canvas_readwrite_for_class(c);
+    /* g_graph_setup_class(c); */
+}
