@@ -18,7 +18,7 @@ put out a "float" as in sliders, toggles, etc. */
 #include "g_all_guis.h"
 #include <math.h>
 
-#ifdef MSW
+#ifdef _WIN32
 #include <io.h>
 #else
 #include <unistd.h>
