@@ -28,6 +28,10 @@
 #include <string.h>
 #include <portaudio.h>
 
+#ifdef _MSC_VER
+#define snprintf sprintf_s
+#endif
+
 #ifndef _WIN32          /* for the "dup2" workaround -- do we still need it? */
 #include <unistd.h>
 #endif
@@ -40,11 +44,14 @@
 # include <stddef.h>        /* BSDs for example */
 #endif                      /* end alloca() ifdef nonsense */
 
-
-#if defined(__APPLE__)
+#if 1
 #define FAKEBLOCKING
 #endif
 
+#if defined (FAKEBLOCKING) && defined(_WIN32)
+#include <windows.h>    /* for Sleep() */
+#endif
+
 /* define this to enable thread signaling instead of polling */
 /* #define THREADSIGNAL */
 
@@ -65,10 +72,10 @@ static int pa_dio_error;
 
 #ifdef FAKEBLOCKING
 #include "s_audio_paring.h"
-static float *pa_outbuf;
-static sys_ringbuf pa_outring;
-static float *pa_inbuf;
-static sys_ringbuf pa_inring;
+static PA_VOLATILE char *pa_outbuf;
+static PA_VOLATILE sys_ringbuf pa_outring;
+static PA_VOLATILE char *pa_inbuf;
+static PA_VOLATILE sys_ringbuf pa_inring;
 #ifdef THREADSIGNAL
 #include <pthread.h>
 pthread_mutex_t pa_mutex;
@@ -76,33 +83,35 @@ pthread_cond_t pa_sem;
 #endif /* THREADSIGNAL */
 #endif  /* FAKEBLOCKING */
 
-
-static void pa_init(void)
+static void pa_init(void)        /* Initialize PortAudio  */
 {
     static int initialized;
     if (!initialized)
     {
-#ifndef _WIN32
-        /* Initialize PortAudio  */
-        /* for some reason Pa_Initialize(0 closes file descriptor 1.
-        As a workaround, dup it to another number and dup2 it back
-        afterward. */
+#ifdef __APPLE__
+        /* for some reason, on the Mac Pa_Initialize() closes file descriptor
+        1 (standard output) As a workaround, dup it to another number and dup2
+        it back afterward. */
         int newfd = dup(1);
+        int another = open("/dev/null", 0);
+        dup2(another, 1);
         int err = Pa_Initialize();
+        close(1);
+        close(another);
         if (newfd >= 0)
         {
+            fflush(stdout);
             dup2(newfd, 1);
             close(newfd);
         }
 #else
         int err = Pa_Initialize();
 #endif
+
+
         if ( err != paNoError ) 
         {
-            fprintf( stderr,
-                "Error number %d occured initializing portaudio\n",
-                err); 
-            fprintf( stderr, "Error message: %s\n", Pa_GetErrorText( err ) );
+            post("Error opening audio: %s", err, Pa_GetErrorText(err));
             return;
         }
         initialized = 1;
@@ -119,7 +128,7 @@ static int pa_lowlevel_callback(const void *inputBuffer,
     float *fbuf, *fp2, *fp3, *soundiop;
     if (nframes % DEFDACBLKSIZE)
     {
-        fprintf(stderr, "portaudio: nframes %ld not a multiple of blocksize %d\n",
+        post("warning: audio nframes %ld not a multiple of blocksize %d",
             nframes, (int)DEFDACBLKSIZE);
         nframes -= (nframes % DEFDACBLKSIZE);
     }
@@ -173,25 +182,25 @@ static int pa_fifo_callback(const void *inputBuffer,
     float *fbuf;
 
 #if CHECKFIFOS
-    if (pa_inchans * sys_ringbuf_GetReadAvailable(&pa_outring) !=   
-        pa_outchans * sys_ringbuf_GetWriteAvailable(&pa_inring))
-            fprintf(stderr, "warning: in and out rings unequal (%d, %d)\n",
-                sys_ringbuf_GetReadAvailable(&pa_outring),
-                 sys_ringbuf_GetWriteAvailable(&pa_inring));
+    if (pa_inchans * sys_ringbuf_getreadavailable(&pa_outring) !=   
+        pa_outchans * sys_ringbuf_getwriteavailable(&pa_inring))
+            post("warning: in and out rings unequal (%d, %d)",
+                sys_ringbuf_getreadavailable(&pa_outring),
+                 sys_ringbuf_getwriteavailable(&pa_inring));
 #endif
-    fiforoom = sys_ringbuf_GetReadAvailable(&pa_outring);
+    fiforoom = sys_ringbuf_getreadavailable(&pa_outring);
     if ((unsigned)fiforoom >= nframes*pa_outchans*sizeof(float))
     {
         if (outputBuffer)
-            sys_ringbuf_Read(&pa_outring, outputBuffer,
-                nframes*pa_outchans*sizeof(float));
+            sys_ringbuf_read(&pa_outring, outputBuffer,
+                nframes*pa_outchans*sizeof(float), pa_outbuf);
         else if (pa_outchans)
-            fprintf(stderr, "no outputBuffer but output channels\n");
+            post("audio error: no outputBuffer but output channels");
         if (inputBuffer)
-            sys_ringbuf_Write(&pa_inring, inputBuffer,
-                nframes*pa_inchans*sizeof(float));
+            sys_ringbuf_write(&pa_inring, inputBuffer,
+                nframes*pa_inchans*sizeof(float), pa_inbuf);
         else if (pa_inchans)
-            fprintf(stderr, "no inputBuffer but input channels\n");
+            post("audio error: no inputBuffer but input channels");
     }
     else
     {        /* PD could not keep up; generate zeros */
@@ -231,15 +240,20 @@ PaError pa_open_callback(double sampleRate, int inchannels, int outchannels,
     instreamparams.device = indeviceno;
     instreamparams.channelCount = inchannels;
     instreamparams.sampleFormat = paFloat32;
-    instreamparams.suggestedLatency = nbuffers*framesperbuf/sampleRate;
     instreamparams.hostApiSpecificStreamInfo = 0;
     
     outstreamparams.device = outdeviceno;
     outstreamparams.channelCount = outchannels;
     outstreamparams.sampleFormat = paFloat32;
-    outstreamparams.suggestedLatency = nbuffers*framesperbuf/sampleRate;;
     outstreamparams.hostApiSpecificStreamInfo = 0;
 
+#ifdef FAKEBLOCKING
+    instreamparams.suggestedLatency = outstreamparams.suggestedLatency = 0;
+#else
+    instreamparams.suggestedLatency = outstreamparams.suggestedLatency = 
+        nbuffers*framesperbuf/sampleRate;
+#endif /* FAKEBLOCKING */
+
     if( inchannels>0 && indeviceno >= 0)
         p_instreamparams=&instreamparams;
     if( outchannels>0 && outdeviceno >= 0)
@@ -300,7 +314,8 @@ PaError pa_open_callback(double sampleRate, int inchannels, int outchannels,
     err = Pa_StartStream(pa_stream);
     if (err != paNoError)
     {
-        fprintf(stderr, "Pa_StartStream failed; closing audio stream...\n");
+        post("error opening failed; closing audio stream: %s",
+            Pa_GetErrorText(err));
         pa_close_audio();
         goto error;
     }
@@ -385,9 +400,9 @@ int pa_open_audio(int inchans, int outchans, int rate, t_sample *soundin,
 
 #ifdef FAKEBLOCKING
     if (pa_inbuf)
-        free(pa_inbuf), pa_inbuf = 0;
+        free((char *)pa_inbuf), pa_inbuf = 0;
     if (pa_outbuf)
-        free(pa_outbuf), pa_outbuf = 0;
+        free((char *)pa_outbuf), pa_outbuf = 0;
 #endif
 
     if (! inchans && !outchans)
@@ -405,14 +420,14 @@ int pa_open_audio(int inchans, int outchans, int rate, t_sample *soundin,
         if (pa_inchans)
         {
             pa_inbuf = malloc(nbuffers*framesperbuf*pa_inchans*sizeof(float));
-            sys_ringbuf_Init(&pa_inring,
+            sys_ringbuf_init(&pa_inring,
                 nbuffers*framesperbuf*pa_inchans*sizeof(float), pa_inbuf,
                     nbuffers*framesperbuf*pa_inchans*sizeof(float));
         }
         if (pa_outchans)
         {
             pa_outbuf = malloc(nbuffers*framesperbuf*pa_outchans*sizeof(float));
-            sys_ringbuf_Init(&pa_outring,
+            sys_ringbuf_init(&pa_outring,
                 nbuffers*framesperbuf*pa_outchans*sizeof(float), pa_outbuf, 0);
         }
         err = pa_open_callback(rate, inchans, outchans,
@@ -426,9 +441,7 @@ int pa_open_audio(int inchans, int outchans, int rate, t_sample *soundin,
     pa_nbuffers = nbuffers;
     if ( err != paNoError ) 
     {
-        fprintf(stderr, "Error number %d opening portaudio stream\n",
-            err); 
-        fprintf( stderr, "Error message: %s\n", Pa_GetErrorText( err ) );
+        post("Error opening audio: %s", Pa_GetErrorText(err));
         /* Pa_Terminate(); */
         return (1);
     }
@@ -447,9 +460,9 @@ void pa_close_audio( void)
     pa_stream = 0;
 #ifdef FAKEBLOCKING
     if (pa_inbuf)
-        free(pa_inbuf), pa_inbuf = 0;
+        free((char *)pa_inbuf), pa_inbuf = 0;
     if (pa_outbuf)
-        free(pa_outbuf), pa_outbuf = 0;
+        free((char *)pa_outbuf), pa_outbuf = 0;
 #endif
 }
 
@@ -474,7 +487,7 @@ int pa_send_dacs(void)
 #ifdef THREADSIGNAL
         pthread_mutex_lock(&pa_mutex);
 #endif
-        while (sys_ringbuf_GetWriteAvailable(&pa_outring) <
+        while (sys_ringbuf_getwriteavailable(&pa_outring) <
             (long)(sys_outchannels * DEFDACBLKSIZE * sizeof(float)))
         {
             rtnval = SENDDACS_SLEPT;
@@ -500,15 +513,15 @@ int pa_send_dacs(void)
                 for (k = 0, fp3 = fp2; k < DEFDACBLKSIZE;
                     k++, fp++, fp3 += sys_outchannels)
                         *fp3 = *fp;
-        sys_ringbuf_Write(&pa_outring, conversionbuf,
-            sys_outchannels*(DEFDACBLKSIZE*sizeof(float)));
+        sys_ringbuf_write(&pa_outring, conversionbuf,
+            sys_outchannels*(DEFDACBLKSIZE*sizeof(float)), pa_outbuf);
     }
     if (sys_inchannels)    /* if there is input sync on it */
     {
 #ifdef THREADSIGNAL
         pthread_mutex_lock(&pa_mutex);
 #endif
-        while (sys_ringbuf_GetReadAvailable(&pa_inring) <
+        while (sys_ringbuf_getreadavailable(&pa_inring) <
             (long)(sys_inchannels * DEFDACBLKSIZE * sizeof(float)))
         {
             rtnval = SENDDACS_SLEPT;
@@ -528,8 +541,8 @@ int pa_send_dacs(void)
     }
     if (sys_inchannels)
     {
-        sys_ringbuf_Read(&pa_inring, conversionbuf,
-            sys_inchannels*(DEFDACBLKSIZE*sizeof(float)));
+        sys_ringbuf_read(&pa_inring, conversionbuf,
+            sys_inchannels*(DEFDACBLKSIZE*sizeof(float)), pa_inbuf);
         for (j = 0, fp = sys_soundin, fp2 = conversionbuf;
             j < sys_inchannels; j++, fp2++)
                 for (k = 0, fp3 = fp2; k < DEFDACBLKSIZE;
@@ -577,45 +590,6 @@ int pa_send_dacs(void)
     return (rtnval);
 }
 
-void pa_listdevs(void)     /* lifted from pa_devs.c in portaudio */
-{
-    int      i,j;
-    int      numDevices;
-    const    PaDeviceInfo *pdi;
-    PaError  err;
-    pa_init();
-    numDevices = Pa_GetDeviceCount();
-    if( numDevices < 0 )
-    {
-        fprintf(stderr, "ERROR: Pa_GetDeviceCount returned %d\n", numDevices );
-        err = numDevices;
-        goto error;
-    }
-    fprintf(stderr, "Audio Devices:\n");
-    for( i=0; i<numDevices; i++ )
-    {
-        pdi = Pa_GetDeviceInfo( i );
-        fprintf(stderr, "device %d:", i+1 );
-        fprintf(stderr, " %s;", pdi->name );
-        fprintf(stderr, "%d inputs, ", pdi->maxInputChannels  );
-        fprintf(stderr, "%d outputs", pdi->maxOutputChannels  );
-        if ( i == Pa_GetDefaultInputDevice() )
-            fprintf(stderr, " (Default Input)");
-        if ( i == Pa_GetDefaultOutputDevice() )
-            fprintf(stderr, " (Default Output)");
-        fprintf(stderr, "\n");
-    }
-
-    fprintf(stderr, "\n");
-    return;
-
-error:
-    fprintf( stderr, "An error occured while using the portaudio stream\n" ); 
-    fprintf( stderr, "Error number: %d\n", err );
-    fprintf( stderr, "Error message: %s\n", Pa_GetErrorText( err ) );
-
-}
-
     /* scanning for devices */
 void pa_getdevs(char *indevlist, int *nindevs,
     char *outdevlist, int *noutdevs, int *canmulti, 
@@ -631,16 +605,33 @@ void pa_getdevs(char *indevlist, int *nindevs,
         const PaDeviceInfo *pdi = Pa_GetDeviceInfo(i);
         if (pdi->maxInputChannels > 0 && nin < maxndev)
         {
-            sprintf(indevlist + nin * devdescsize, "(%d)%s",
-                pdi->hostApi,pdi->name);
-            /* strcpy(indevlist + nin * devdescsize, pdi->name); */
+                /* LATER figure out how to get API name correctly */
+            snprintf(indevlist + nin * devdescsize, devdescsize,
+#ifdef _WIN32
+     "%s:%s", (pdi->hostApi == 0 ? "MMIO" : (pdi->hostApi == 1 ? "ASIO" : "?")),
+#else
+#ifdef __APPLE__
+             "%s",
+#else
+            "(%d) %s", pdi->hostApi,
+#endif
+#endif
+                pdi->name);
             nin++;
         }
         if (pdi->maxOutputChannels > 0 && nout < maxndev)
         {
-            sprintf(outdevlist + nout * devdescsize, "(%d)%s",
-                pdi->hostApi,pdi->name);
-            /* strcpy(outdevlist + nout * devdescsize, pdi->name); */
+            snprintf(outdevlist + nout * devdescsize, devdescsize,
+#ifdef _WIN32
+     "%s:%s", (pdi->hostApi == 0 ? "MMIO" : (pdi->hostApi == 1 ? "ASIO" : "?")),
+#else
+#ifdef __APPLE__
+             "%s",
+#else
+            "(%d) %s", pdi->hostApi,
+#endif
+#endif
+                pdi->name);
             nout++;
         }
     }
