@@ -274,7 +274,10 @@ int sys_main(int argc, char **argv)
 # ifdef _MSC_VER /* MS Visual Studio */
     _set_fmode( _O_BINARY );
 # else  /* MinGW */
-    int _fmode = _O_BINARY;
+    {
+        extern int _fmode;
+        _fmode = _O_BINARY;
+    }
 # endif /* _MSC_VER */
 #endif  /* WIN32 */
     pd_init();                                  /* start the message system */
@@ -306,7 +309,8 @@ int sys_main(int argc, char **argv)
     {
             /* open audio and MIDI */
         sys_reopen_midi();
-        sys_reopen_audio();
+        if (audio_shouldkeepopen())
+            sys_reopen_audio();
             /* run scheduler until it quits */
         return (m_mainloop());
     }
